@@ -23,6 +23,11 @@ config = {
     },
 
     ## Backend stuff ##
+
+    # This is the list of enabled backends (so we can turn one off globally)
+    #"fmn.backends": ['email', 'irc', 'android'],
+    "fmn.backends": ['email', 'irc'],
+
     # Email
     "fmn.email.mailserver": "127.0.0.1:25",
     "fmn.email.from_address": "notifications@fedoraproject.org",
@@ -47,6 +52,7 @@ config = {
     "endpoints": {
         "fmn.%s" % hostname: [
             "tcp://127.0.0.1:3041",
+            "tcp://127.0.0.1:3042",
         ],
     },
     "logging": dict(
